import numpy as np
from scipy.interpolate import RectBivariateSpline
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import nupropagator.flux.Flux as fff
name = ['Numu_H3a+KM_E2.DAT','Numu_H3a+QGS_E2.dat','Numu_H3a+SIBYLL_E2.dat']
model = ['H3a+KM','H3a+QGS','H3a+SIBYLL']
color = ['k','g','r']
X = np.logspace(1,8,100)
Y = np.linspace(0,1,100)
x,y = np.meshgrid(X,Y)
fig, ax = plt.subplots(subplot_kw={'projection': '3d'})
x = np.log10(x)
y = y
for i in range(len(name)): 
    fff1 = fff.Flux('KM',name[i],3)
    a = fff1.get_rect_splain()
    z = np.log10(fff1.I3(Y, X))
    ax.plot_wireframe(x,y,z, color=color[i],label= model[i])
ax.set_xlabel(r'lg(E_{\nu}/GeV)')
ax.set_ylabel(r'\cos\theta')
ax.set_zlabel(r'$\lg(F_{\nu}/cm^2/srad/s/GeV)$')
plt.legend()
fig.tight_layout()
plt.savefig('/home/vlad/flux_intepol.pdf')
plt.show()




import matplotlib
import XsDIS as xs
from NeutrinoNucleonXsDIS import NeutrinoNucleonXsDIS
from pdg_constants import *
xs.nucleonstructurefunctions(1,1,1,1,1,1,1,1,1)
import matplotlib.pyplot as plt
from matplotlib.colors import BoundaryNorm
from matplotlib.ticker import MaxNLocator
import Earth as earth
import Flux as fff
import cross as cs
import Global as g
import FinVtx as fin
import numpy as np
import time as time
from scipy.integrate import*
s=cs.Cross_section(model='CTEQ6.5M',pdg=14,n_tt=1)
s1=cs.Cross_section(model='CTEQ6.5M',pdg=14,n_tt=2)
#import Kinematic as kin
#import Particle as par
import zf as zf1
def initialize(dis,target):
    dis.init_structure_functions_model(2)
    dis.init_neutrino(14)
    dis.init_charged_lepton_polarization(0)
    dis.init_nucleon(target)
    dis.init_Rfunction_model(2,1)
    dis.init_Wcut(1.2*GeV)
    dis.init_FL_model(2)
    dis.init_Q2_fixing(0)
def proba(i,cos,z,nu):
    k=np.int_(z/10.)
    j=np.int_(100*cos+100)
    return nu.prob[i][j][k]
def proba1(i,cos,z,nu):
    k=np.int_(z/10.)
    j=np.int_(100*cos+100)
    return nu.prob1[i][j][k]

def den(z,earth):
    return earth.get_density(g.R_E-z,'PREM')
def fl1(i,cos,f1,e):
    energy=e[i]
    if energy<10**12 and energy>=10. and cos<=1 and cos>=0:
        return f1.get_3D_splain_points(energy,cos)
    else:
        return 0

def fl11(i,cos,z,f1,e):
    a=1-z/g.R_E
    #cos1=((1-2*a*cos+a*a*cos*cos)/(1-2*a*cos+a*a))**0.5
    cos1=(1-a*a+a*a*cos*cos)**0.5
    return fl1(i,cos1,f1,e)


def aaaaaa1(i,cos,z,nu,f1,e):
    a=1-z/g.R_E
    #cos1=((1-2*a*cos+a*a*cos*cos)/(1-2*a*cos+a*a))**0.5
    cos1=(1-a*a+a*a*cos*cos)**0.5
    return fl1(i,cos1,f1,e)*proba(i,cos,z,nu)
def aaaaaa2(i,cos,z,nu,f1,e):
    a=1-z/g.R_E
    #cos1=((1-2*a*cos+a*a*cos*cos)/(1-2*a*cos+a*a))**0.5
    cos1=(1-a*a+a*a*cos*cos)**0.5
    return fl1(i,cos1,f1,e)*proba1(i,cos,z,nu)

def maaaaaa1(i,cos,z,nu,f1,e,kk):
    b=quad(lambda y: aaaaaa1(i-int(np.log10(1-np.exp(y))),cos,z,nu,f1,e)*xs.d2sdiscc_dxdy(e[i],kk,np.exp(y))*10**(-38)*np.exp(y),-10,0)
    print(b[0])
    return 1.678*b[0]
def maaaaaa2(i,cos,z,nu,f1,e,kk):
    b=quad(lambda y: aaaaaa2(i-int(np.log10(1-np.exp(y))),cos,z,nu,f1,e)*xs.d2sdiscc_dxdy(e[i],kk,np.exp(y))*10**(-38)*np.exp(y),-10,0)
    print(b[0])
    return 1.678*b[0]

def omegaxy(energy,x,y):
    return xs.d2sdiscc_dxdy(energy,x,y)**18*1.678*10**(-38)/((10*s.cross_sectioncc(energy)+8*s1.cross_sectioncc(energy)))

def omegay(energy,y):
    b=quad(lambda x: omegaxy(energy,np.exp(x),y)*np.exp(x),-15*np.log(10),0,epsrel=10**(-4))
    return b[0]
def omegax(energy,x):
    b=quad(lambda y: omegaxy(energy,x,np.exp(y))*np.exp(y),-10,0,epsrel=10**(-4))
    return b[0]

def omega(energy):
    c=quad(lambda y:omegay(energy/(1-np.exp(y)),np.exp(y))*np.exp(y),-15,0 )
    return c[0]
def omega1(energy,cos,z,nu,f1,e):
    c=dblquad(lambda y,x: aaaaaa1(int(np.log10(energy/(1-np.exp(y)))),cos,z,nu,f1,e)*omegaxy(energy/(1-np.exp(y)),np.exp(x),np.exp(y))*np.exp(x+y), -15,0,-10,0,epsrel=10**(-3))
    print('integral ',c[0])
    return c[0]

def lll(ne,ny):
    dis = NeutrinoNucleonXsDIS()
    initialize(dis,2212)
    aaa1=np.zeros([ne,ny])
    aaa3=np.zeros([ne,ny])
    aaa2=np.zeros([ne,ny])
    aaa4=np.zeros([ne,ny])
    bbb1=np.zeros([ne,ny])
    bbb2=np.zeros([ne,ny])
    e=np.array([10**(10*i/float(ne-1)) for i in range(ne)])
    y1=np.logspace(-10,-3,num=ny)
    y2=np.linspace(0.001,1,num=ny)
    for i in range(ne):
        for j in range(ny):
            aaa1[i][j]=omegay(e[i],y1[j])
            print('p ',i,j)
    initialize(dis,2112)
    for i in range(ne):
        for j in range(ny):
            aaa2[i][j]=omegay(e[i],y1[j])
            print('n ',i,j)
    initialize(dis,2212)
    for i in range(ne):
        for j in range(ny):
            aaa3[i][j]=omegay(e[i],y2[j])
            print('p ',i,j)
    initialize(dis,2112)
    for i in range(ne):
        for j in range(ny):
            aaa4[i][j]=omegay(e[i],y2[j])
            print('n ',i,j)

    fil=open('ydis3.dat','w')
    bbb1=aaa1+aaa2
    bbb2=aaa3+aaa4
    for i in range(ne):
        for j in range(ny):
            fil.write(str(e[i])+'\t'+str(y1[j])+'\t'+str(bbb1[i][j])+'\n')
    for i in range(ne):
        for j in range(ny):
            fil.write(str(e[i])+'\t'+str(y2[j])+'\t'+str(bbb2[i][j])+'\n')

    fil.close()

    return y1,bbb1

def lllx(ne,nx):
    dis = NeutrinoNucleonXsDIS()
    initialize(dis,2212)
    aaa1=np.zeros([ne,nx])
    aaa2=np.zeros([ne,nx])
    bbb=np.zeros([ne,nx])
    e=np.array([10**(10*i/float(ne-1)) for i in range(ne)])
    x=np.array([10**(15*(i/float(nx-1)-1)) for i in range(nx) ])
    for i in range(ne):
        for j in range(nx):
            aaa1[i][j]=omegax(e[i],x[j])
            print('p ',i,j)
    initialize(dis,2112)
    for i in range(ne):
        for j in range(nx):
            aaa2[i][j]=omegax(e[i],x[j])
            print('n ',i,j)
    fil=open('xdis.dat','w')
    bbb=aaa1+aaa2
    for i in range(ne):
        for j in range(nx):
            fil.write(str(e[i])+'\t'+str(x[j])+'\t'+str(bbb[i][j])+'\n')
    fil.close()

    return x,bbb


def lllm():
    fig=plt.figure()
    for j in range(10):
        a=lll(10**(j))
        plt.scatter(a[0],a[1],label='E = '+str('{:1.1e}'.format(10**j)))
    plt.xlabel('y')
    plt.ylabel('distribution')
    plt.title(r'$\frac{d\sigma}{dy}(E,y)$')
    plt.grid(True)
    plt.xscale('log')
    plt.legend()
    plt.show()

def llll():
    N=1000
    dis = NeutrinoNucleonXsDIS()
    initialize(dis,2212)
    fig=plt.figure()
    bbb=np.zeros(11)
    f1=fff.Flux('KM','flux_data/Numu_H3a+KM_E2.DAT',3)
    aaa=np.zeros([11,21])
    e=np.array([10**(i) for i in range(11)])
    nu=NuPropogator(11,21,1000)
    nu.probability1(N,14,e)
#    y=np.array([10**(10*(i/100.-1)) for i in range(101)])
    for i in range(11):
        for j in range(21):
            a=time.time()
            cos=(-10+j)/10.
            aaa[i][j]=omega1(e[i],cos,1000,nu,f1,e)
            print(time.time()-a)
    for i in range(11):
        bbb[i]=aaa[i].sum()*0.1
    fil = open('muondata1000.dat', 'w')
    for i in range(11):
        fil.write(str(e[i])+'t'+str(bbb[i])+'\n')
    fil.close()


class NuPropogator:
    def __init__(self,m,n,r):
            self.n=n #number of bins for cos\theta
            self.m=m #number of points for energy
            self.r=r #number of points for z
            self.prob=np.zeros([m,n,r])
            self.prob1=np.zeros([m,n,r])
            self.depth=np.zeros([n,r])
            self.zf=np.zeros([m,n,r])
            print('Propogator is ready')
        
    def probability(self,N,flavor,energy,v1,v2):
        a=0
        s=cs.Cross_section(model='CTEQ6.5M',pdg=14,n_tt=1)
        s1=cs.Cross_section(model='CTEQ6.5M',pdg=14,n_tt=2)
        erth=earth.Earth(self.r,'PREM')
        dep=np.zeros([self.n,self.r])
        for j in range(self.n):
            x=time.time()
            dep[j]=erth.column_depth(v1,v2,N)
            a=a+time.time()-x
            print(j,time.time()-x)
        print(a)
        for i in range(self.m):
            self.prob[i]=np.exp(-(s.tot(energy[i])+s1.tot(energy[i]))*g.cm**2*g.N_A*dep/g.g)
    
#    def get_product(self,flavor,P):
#        s=cc.CC()
#        r=kin.Kinematic()
#        final=r.cc_scatter(flavor,P,s)
#        print('lepton: ',final[0])
#        print('hadron',final[1])
        
        
    def probability1(self,N,flavor,energy):
        a=0
        zz=zf1.zf()
        s=cs.Cross_section(model='CTEQ6.5M',pdg=14,n_tt=1)
        s1=cs.Cross_section(model='CTEQ6.5M',pdg=14,n_tt=2)
        erth=earth.Earth(self.r,'PREM')
        dep=np.zeros([self.n,self.r])
        x2=np.zeros(self.r)
        y2=np.zeros(self.r)
        z2=np.array([g.R_E-10*g.m*i for i in range(self.r)])
        for j in range(self.n):
            x=time.time()
            cos=(-(self.n-1.)/2+j)/((self.n-1.)/2)
            x1=np.array([g.R_E*(1-cos**2)**0.5 for i in range(self.r)])
            y1=np.zeros(self.r)
            z1=np.array([g.R_E*cos for i in range(self.r)])
            v1=fin.FinVtx(x1,y1,z1)
            v2=fin.FinVtx(x2,y2,z2)
            dep[j]=erth.column_depth(v1,v2,N)
            a=a+time.time()-x
            print(j,time.time()-x)
        print(a)
        self.depth=dep
        for i in range(self.m):
            for j in range(self.n):
                for k in range(self.r):
                    self.zf[i,j,k]=zz.f(g.g*dep[j][k]/g.cm**2,energy[i])
                print(i,j)


        for i in range(self.m):
            self.prob[i]=np.exp(-(s.tot(energy[i])+s1.tot(energy[i]))*g.cm**2*g.N_A*dep/g.g)
            self.prob1[i]=np.exp(-(s.tot(energy[i])+s1.tot(energy[i]))*(1-self.zf[i])*g.cm**2*g.N_A*dep/g.g)
        
    
    def test(self):
        N=1000
        print('Hello')
        e=np.logspace(1,8,num=self.m)
        self.probability1(N,14,e)
        cos=np.array([(-(self.n-1.)/2+j)/((self.n-1.)/2) for j in range(self.n)])
        for i in range(8):
            fig = plt.figure(figsize=(18,12))
            plt.plot(cos,self.prob[i,:,10],label='without regeneration')
            plt.plot(cos,self.prob1[i,:,10],label='Z-factor regeneration')
            plt.title(' Probability of propogating netrino from surface to z = '+str(100)+'m for E = '+'{:1.1e}'.format(e[i]))
            plt.ylabel(r'$log_{10}(P_{\nu})$')
            plt.xlabel(r'$cos\theta$')
            if(i>=5):
                plt.yscale('log')
            plt.legend()
            plt.grid(True)
            plt.savefig(str(i)+'prob_z.png')
            plt.show()
            plt.close()
    

    def test1(self):
        N=1000
        nuflux=np.zeros([self.m,self.n,self.r])
        print('Hello')
        e=np.array([10**(i/100000.)*g.GeV for i in range(self.m)])
        cos=1.
        x1=np.zeros(self.r)
        x2=np.zeros(self.r)
        y1=np.zeros(self.r)
        y2=np.zeros(self.r)
        z1=np.array([g.R_E for i in range(self.r)])
        z2=np.array([g.R_E-600*g.m*i for i in range(self.r)])
        v1=fin.FinVtx(x1,y1,z1)
        v2=fin.FinVtx(x2,y2,z2)
        self.probability(N,14,e,v1,v2)
        for i in range(10):
            k=10*i
            fig = plt.figure()
            plt.plot(z1-z2,np.log10(self.prob[k,0]),label=r'$E_{\nu}$ = '+str('{:1.2e}'.format(10**(k/10.))))
            plt.ylabel(r'$log_{10}(P_{\nu})$')
            plt.xlabel(r'$cos\theta$')
            plt.legend()
            plt.grid(True)
            plt.savefig('probability/probenergy'+str(k)+'.png')
            plt.show()
            plt.close()


    def testnuflux(self,z):
        N=1000
        e=np.logspace(1,8,num=self.m)
        xxx=np.zeros([self.m,self.n])
        xxx1=np.zeros([self.m,self.n])
        xxx2=np.zeros([self.m,self.n])
        print('Hello')
        self.probability1(N,14,e)
        f1=fff.Flux('KM','flux_data/Numu_H3a+KM_E2.DAT',3)
        cos=np.array([(-(self.n-1.)/2+j)/((self.n-1.)/2) for j in range(self.n)])
        z1=np.array([g.R_E for i in range(self.r)])
        z2=np.array([g.R_E-10*g.m*i for i in range(self.r)])
        for i in range((self.m)):
            for j in range((self.n)):
                k=int(z/10.)
                xxx[i][j]=aaaaaa1(i,cos[j],z1[k]-z2[k],self,f1,e)
                xxx1[i][j]=aaaaaa2(i,cos[j],z1[k]-z2[k],self,f1,e)
                xxx2[i][j]=fl11(i,cos[j],z1[k]-z2[k],f1,e)
                print(i,j,xxx[i][j]-xxx1[i][j])
        for i in [0,50,100,150,200]:
            fig = plt.figure(figsize=(18,12))
            plt.plot(e,xxx[:,i]*e*e,label=r'without regeneration $\cos\theta = $'+str(cos[i]))
            plt.plot(e,xxx1[:,i]*e*e,label=r'Z-factor regeneration $\cos\theta = $'+str(cos[i]))
            plt.plot(e,xxx2[:,i]*e*e,label=r'$F_{\nu}(x=0,E)$ for $\cos\theta$ = '+str(cos[i]))
            #plt.title('Flux of netrino in z = '+str(100)+'m for E = '+'{:1.1e}'.format(e[i]))
            plt.title('Flux of netrino in z = '+str(z)+'m' )
            plt.ylabel(r'$F_{\nu}(x,E)\cdot E_{\nu}^2$,$GeV\cdot cm^2\cdot s^{-1}\cdot sr^{-2}$')
            plt.xlabel(r'$E_{\nu}$, GeV')
            #if i>3:
            plt.yscale('log')
            plt.legend()
            plt.xscale('log')
            plt.grid(True)
            plt.savefig('cos='+str(i)+'flux_z'+str(z)+'.png')
            plt.show()
            plt.close()

        fil=open('prop'+str(z)+'.dat','w')
        fil1=open('prop0.dat','w')
        for i in range(self.m):
            for j in range(self.n):
                fil.write(str(e[i])+'\t'+str(cos[j])+'\t'+str(xxx1[i][j])+'\n')
                fil1.write(str(e[i])+'\t'+str(cos[j])+'\t'+str(xxx[i][j])+'\n')
        fil.close()
#        print('heyyyyyy  ',z/1000.)
#        for i in range(10):
#            fig, (ax0) = plt.figure()
#            fig = plt.figure()
#            k=i
#            for j in range(10):
#                t=40*j+10
#                levels = MaxNLocator(nbins=1000).tick_values(np.log10(self.prob[10*i,:,:]).min(),np.log10(self.prob[10*i,:,:]).max())
#                cmap = plt.get_cmap('inferno')
#                norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)
#            im = ax0.pcolormesh(z1-z2,cos, np.log10(self.prob[10*i,:,:]), cmap=cmap, norm=norm)
#            fig.colorbar(im, ax=ax0)
#                plt.scatter(cos,np.log10(xxx[k,:,int(t/10)]),label=r'$h$ = '+str('{:1.2e}'.format(t))+' meters')
#            plt.ylabel(r'$log_{10}(E_{\nu_{mu}})$')
#                ax0.set_title('Probability of propogating netrino from surface to z= '+str(t)+'meters')
#            plt.title('Probability of propogating netrino from surface to h for E= '+str('{:1.1e}'.format(10**(i+1)))+' GeV')
#            plt.ylabel(r'$log_{10}(P_{\nu})$')
#            plt.xlabel(r'$cos\theta$')
#            plt.legend()
#            plt.grid(True)
#            plt.savefig('flux/nuflux'+str(i)+'.png')
#            plt.show()
#            plt.close()

    def testmuflux(self):
        N=1000
        s=cs.Cross_section(model='CTEQ6.5M',pdg=14,n_tt=1)
        s1=cs.Cross_section(model='CTEQ6.5M',pdg=14,n_tt=2)

        dis = NeutrinoNucleonXsDIS()
        dis1 = NeutrinoNucleonXsDIS()
        initialize(dis,2212)
        xxx=np.zeros([self.m,self.n,self.r,101])
        print('Hello')
        e=np.array([10**(i+1)*g.GeV for i in range(self.m)])
        self.probability1(N,14,e)
        f1=fff.Flux('KM','flux_data/Numu_H3a+KM_E2.DAT',3)
        cos=np.array([(-(self.n-1.)/2+j)/((self.n-1.)/2) for j in range(self.n)])
        z1=np.array([g.R_E for i in range(self.r)])
        z2=np.array([g.R_E-10*g.m*i for i in range(self.r)])

        for i in range((self.m)):
            for j in range((self.n)):
                for k in range(self.r):
                    for kk in range(16):
                        xxx[i][j][k][kk]=maaaaaa1(i,cos[j],z1[k]-z2[k],self,f1,e,10**(15*(kk/15.-1)))/((s.cross_sectiontot(e[i])+s1.cross_sectiontot(e[i]))*g.cm**2)
                        xxx[i][j][k][kk]=maaaaaa1(i,cos[j],z1[k]-z2[k],self,f1,e,10**(15*(kk/15.-1)))/((s.cross_sectiontot(e[i])+s1.cross_sectiontot(e[i]))*g.cm**2)
                print(i,j)
#        for i in range(10):
#            fig, (ax0) = plt.figure()
#            fig = plt.figure()
#            k=i
#            for j in range(16):
                
#                levels = MaxNLocator(nbins=1000).tick_values(np.log10(self.prob[10*i,:,:]).min(),np.log10(self.prob[10*i,:,:]).max())
#                cmap = plt.get_cmap('inferno')
#                norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)
#            im = ax0.pcolormesh(z1-z2,cos, np.log10(self.prob[10*i,:,:]), cmap=cmap, norm=norm)
#            fig.colorbar(im, ax=ax0)
#                plt.scatter(cos,np.log10(xxx[k,:,5,j]),label=r'$x$ = '+str('{:1.2e}'.format(10**(15*(j/15.-1)))))
#            plt.ylabel(r'$log_{10}(E_{\nu_{mu}})$')
#                ax0.set_title('Probability of propogating netrino from surface to z= '+str(t)+'meters')
#            plt.title('Probability of propogating netrino from surface to h for E= '+str('{:1.1e}'.format(10**(i+1)))+' GeV')
#            plt.ylabel(r'$log_{10}(P_{\nu})$')
#            plt.xlabel(r'$cos\theta$')
#            plt.legend()
#            plt.grid(True) 
#            plt.savefig('flux/muflux'+str(i)+'.png')
#            plt.show()
#            plt.close()

    def testmuflux1(self):
        N=1000
        s=cs.Cross_section(model='CTEQ6.5M',pdg=14,n_tt=1)
        s1=cs.Cross_section(model='CTEQ6.5M',pdg=14,n_tt=2)
        
        dis = NeutrinoNucleonXsDIS()
        dis1 = NeutrinoNucleonXsDIS()
        initialize(dis,2212)
        xxx=np.zeros([self.m,self.n,self.r,101])
        print('Hello')
        e=np.array([10**(i+1)*g.GeV for i in range(self.m)])
        self.probability1(N,14,e)
        f1=fff.Flux('KM','flux_data/Numu_H3a+KM_E2.DAT',3)
        cos=np.array([(-(self.n-1.)/2+j)/((self.n-1.)/2) for j in range(self.n)])
        z1=np.array([g.R_E for i in range(self.r)])
        z2=np.array([g.R_E-1*g.m*i for i in range(self.r)])
        for i in range((self.m)):
            for j in range((self.n)):
                for k in range(self.r):
                    for kk in range(11):
                        xxx[i][j][k][kk]=maaaaaa1(i,cos[j],z1[k]-z2[k],self,f1,e,10**(15*(kk/100.-1)))/(s.cross_sectiontot(e[i])+s1.cross_sectiontot(e[i]))
                print('cccccccccccccccccccc',i,j)
        for i in range(10):
            fig = plt.figure()
            k=i
            for j in range(10):
                t=40*j+10
                plt.scatter(cos,np.log10(xxx[k,:,int(t/10)]),label=r'$h$ = '+str('{:1.2e}'.format(t))+' meters') 
            plt.title('Probability of propogating netrino from surface to h for E= '+str('{:1.1e}'.format(10**(i+1)))+' GeV')
            plt.ylabel(r'$log_{10}(P_{\nu})$')
            plt.xlabel(r'$cos\theta$')
            plt.legend()
            plt.grid(True)
            plt.savefig('flux/muflux'+str(i)+'.png')
            plt.show()
            plt.close()




nu=NuPropogator(8,201,200)
#nu.test()
#nu.prob
#nu.testnuflux(1000000)
nu.testnuflux(1000)
#lll(111,101)



from types import SimpleNamespace
opts = SimpleNamespace()
opts.neutrinoPrimary_energy_mode = 'random'
opts.neutrinoPrimary_direction_mode = 'r'
opts.neutrinoPrimary_target_mode = 'random'
opts.neutrinoPrimary_current_mode = 'random'
opts.neutrinoPrimary_pdgid = 14
opts.neutrinoPrimary_pdf_model = 'CT10nlo'
opts.N_event = 1
opts.vegas_neval = 1000 
import time as time
import nupropagator.nugen as  nugen
import numpy as np
import nupropagator.NuPropagator as nuprop
import nupropagator.Global.Global as g
nu = nugen.NuGen()
#M = np.array([1,10]) 
M = np.array([1,10,100,1000,3000,7000,10000]) 
nuprop = nuprop.NuPropagator(opts)
print('start')
#N = len(nuprop.spectra[0])
N = 10
tt1 = np.zeros(N)
tt2 = np.zeros(N)
tt1_t = np.zeros(len(M))
tt2_t = np.zeros(len(M))
n1 = 0
n2 = 0

nuprop.set_final_vertex(np.array([0,0,g.R_E - 1000]))
for m in range(len(M)):
    for i in range(N):
        #print('EVENT ', i)
        print(m,i)
        nuprop.set_event_number(i)
        info_drag = nuprop.get_dragging()
        opts.neutrinoPrimary_energy_GeV = info_drag[1] 
        opts.neutrinoPrimary_direction = info_drag[2]
        tt1[i] = time.time()
        opts.N_event = 1
        for k in range(M[m]):
            n1 = nu.get_event(opts)# 0.03s per event
        tt1[i] = (time.time()-tt1[i])/M[m]
        tt2[i] = time.time()
        if M[m]>=1000:
            opts.vegas_neval = M[m]
        opts.N_event = 1000
        n2 = nu.get_event_fix_en(opts)# 0.17s per 1000 event
        tt2[i] = (time.time()-tt2[i])/opts.N_event
    tt1_t[m] = tt1.mean()
    tt2_t[m] = tt2.mean()
import matplotlib.pyplot as plt
fig = plt.figure(figsize = (18,12))
plt.scatter(M,tt1_t, label = 'monocall')
plt.scatter(M,tt2_t,label ='multicall')
plt.xscale('log')
plt.xlabel('number of events')
plt.ylabel('CPU time, s')
plt.grid(True)
plt.legend()
plt.savefig('/home/vlad/CPU_time.pdf')
plt.show()



import numpy as np
from nupropagator.cross_section import cross as cs
from nupropagator.cross_section import cross_ground as cg
from nudisxs.disxs import *
from scipy.spatial.transform import Rotation as R
from nudisxs.integrand_2d import*
class NuGen:
    def __init__(self):
        self.nu_energy_GeV = 0.
        self.nu_direction  = [0.,0.,1.]
        self.target        = -1 # not possible value. must be configured
        self.current = {-1: 'nc', 1: 'cc'}
        self.target_dict = {2212: 'proton', 2112: 'neutron'}
        self.dis = disxs()
        self.integ = integrand_2dxy()
    
    def configure(self,opts):
        self.configure_simulation_mode(opts)
        self.configure_target(opts)
        self.configure_neutrino(opts)
        import logging
        self.log = logging.getLogger('nupropagator.NuGen')
        #self.log.info(f'neutrino energy mode={self.nu_energy_mode}')
        #self.log.info(f'neutrino direction mode={self.nu_direction_mode}')
        #self.log.info(f'target mode={self.target_mode}')
        #self.log.info(f'neutrino pdg ={self.nu_pdg}')

    def configure_simulation_mode(self,opts):
        self.nu_energy_mode = opts.neutrinoPrimary_energy_mode
        self.nu_direction_mode = opts.neutrinoPrimary_direction_mode
        self.target_mode = opts.neutrinoPrimary_target_mode
        self.current_mode = opts.neutrinoPrimary_current_mode

    def configure_neutrino(self,opts):
        self.nu_pdg = opts.neutrinoPrimary_pdgid
        self.cs_p = cs.Cross_section(self.pdf_model,14,2212)
        self.cs_n = cs.Cross_section(self.pdf_model,14,2112)
        self.cs = {2112: self.cs_n, 2212: self.cs_p}
        self.cross_ground_p_NC = cg.cross_ground(self.pdf_model,'nc',self.nu_pdg,2212)
        self.cross_ground_n_NC = cg.cross_ground(self.pdf_model,'nc',self.nu_pdg,2112)
        self.cross_ground_p_CC = cg.cross_ground(self.pdf_model,'cc',self.nu_pdg,2212)
        self.cross_ground_n_CC = cg.cross_ground(self.pdf_model,'cc',self.nu_pdg,2112)
        self.cross_ground = {(1,2112): self.cross_ground_n_CC.f, (1,2212): self.cross_ground_p_CC.f,
                 (-1,2112): self.cross_ground_n_NC.f, (-1,2212): self.cross_ground_p_NC.f}
        self.xsec_xy = {1: self.dis.xs_cc, -1: self.dis.xs_nc}
        self.integ.dis.init_neutrino(self.nu_pdg)
        self.dis.init_neutrino(self.nu_pdg)
        if self.nu_energy_mode != 'random':
            self.nu_energy_GeV = opts.neutrinoPrimary_energy_GeV
        else: self.nu_energy_GeV = opts.neutrinoPrimary_energy_GeV

        if self.nu_direction_mode != 'random':
            self.nu_direction = opts.neutrinoPrimary_direction
        else: self.nu_direction = opts.neutrinoPrimary_direction

    def configure_target(self,opts):#FIXME
        self.pdf_model = opts.neutrinoPrimary_pdf_model
        self.dis.init_pdf(self.pdf_model)
        if self.target_mode != 'random':
            self.target = opts.neutrinoPrimary_target

    def get_neutrino_direction(self):#FIXME
        if self.nu_direction_mode == 'random':
            nu_phi = 2*np.pi*np.random.random()
            nu_cos = 2*np.random.random()-1
            nu_sin = (1-nu_cos**2)**0.5
        else:
            nu_phi = np.arctan2(self.nu_direction[1],self.nu_direction[0])
            nu_cos = self.nu_direction[2]
            nu_sin = np.sqrt(self.nu_direction[0]**2+self.nu_direction[1])

        return nu_cos, nu_sin, nu_phi

    def get_neutrino_energy(self):
        if self.nu_energy_mode != 'random':
            return self.nu_energy_GeV
        else:
            # select random
            return self.nu_energy_GeV # FIX')

    def get_target(self):
        # get nucleon FIXME. Should be more generic: not only for water
        if self.target_mode == 'random':
            a = np.random.random()
            if a<=4./9:
                self.target = 2112
                self.dis.init_target('neutron')
            else:
                self.target = 2212
                self.dis.init_target('proton')
        self.dis.init_target(self.target_dict[self.target])
        self.integ.dis.init_target(self.target_dict[self.target])
    
    def get_target_multi(self,N):
        a = np.random.random(N)
        b = np.int_(a<=4./9)*2112+np.int_(a>4./9)*2212
        return b

    def get_current(self):# FIXME
        if self.current_mode == 'random':
            a = np.random.random()
            mode = np.int_(a<=self.cs[self.target].ratio_nc(self.nu_energy_GeV))*(-1)
            mode = mode + np.int_(a>self.cs[self.target].ratio_nc(self.nu_energy_GeV))*(1)
            #if self.target == 2212:
            #    mode = np.int_(a <= self.cs_p.ratio_nc(self.nu_energy_GeV))*(-1)
            #    mode = mode + np.int_(a> self.cs_p.ratio_nc(self.nu_energy_GeV))*1
            #if self.target == 2112:
            #    mode = np.int_(a <= self.cs_n.ratio_nc(self.nu_energy_GeV))*(-1)
            #    mode = mode + np.int_(a> self.cs_n.ratio_nc(self.nu_energy_GeV))*1
        else:
            mode = 0
        self.dis.init_current(self.current[mode])
        self.integ.dis.init_current(self.current[mode])
        return mode

    def get_xy_Bjorken_old(self,mode):
        x=y=-1.0 # impossible values
        self.dis.init_current(self.current[mode])
        
        def get_random(x,y,crossing,diff_xsec_xy):
            c =  crossing(self.nu_energy_GeV)*np.random.random()
            xsec = diff_xsec_xy(self.nu_energy_GeV,x,y)
            if(c-xsec >= 0): 
              return True 

        while(True):
            x = np.random.random()
            y = np.random.random()
            if get_random(x,y,self.cross_ground[(mode,self.target)],self.xsec_xy[mode]):
               break 
        return x,y
    
    def get_xy_Bjorken_new(self,mode,opts):#FIXME
        x = []
        y = []
        weight = []
        self.integ.init_vegas_integrator(opts)
        self.integ.calculate(self.nu_energy_GeV)
        i = 0
        for Bj,wgt in self.integ.integrator.random():
            i = i+1
            x.append(Bj[0])
            y.append(Bj[1])
            weight.append(wgt)
        x = np.array(x)
        y = np.array(y)
        weight = np.array(weight)
        xsection = np.zeros(len(x))
        xsection = self.integ.dis.xsdis_as_array(self.nu_energy_GeV,x,y,xsection,len(x))
        return np.array(x),np.array(y),np.array(weight),np.array(xsection),i
    '''
    def get_xy_Bjorken_grond_algo(self,mode):
        x=y=-1.0 # impossible values
        while(True):
            x = np.random.random()
            y = np.random.random()
            if mode == -1:
                self.dis.init_current('nc')
                rn_p_NC =  self.cross_ground_p_NC.f(self.nu_energy_GeV)*np.random.random()
                rn_n_NC =  self.cross_ground_n_NC.f(self.nu_energy_GeV)*np.random.random()
                rn_NC = rn_p_NC*np.int_(self.target == 2212) + rn_n_NC*np.int_(self.target == 2112)
                xsec = self.dis.xs_nc(self.nu_energy_GeV,x,y)
                if(rn_NC-xsec >= 0): break
            if mode == 1:
                self.dis.init_current('cc')
                rn_p_CC =  self.cross_ground_p_CC.f(self.nu_energy_GeV)*np.random.random()
                rn_n_CC =  self.cross_ground_n_CC.f(self.nu_energy_GeV)*np.random.random()
                xsec = self.dis.xs_cc(self.nu_energy_GeV,x,y)
                rn_CC = rn_p_CC*np.int_(self.target == 2212) + rn_n_CC*np.int_(self.target == 2112)
                if(rn_CC-xsec >= 0):
                    break
        return x,y
    '''
    def rotate_vectors(self,p,nu_phi,nu_cos,N):
        if N ==1:
            phi_d = np.tile([0,0,nu_phi],(N))
            theta_d = np.tile([0,np.arccos(nu_cos),0],(N))
        else:
            phi_d = np.tile([0,0,nu_phi],(N,1))
            theta_d = np.tile([0,np.arccos(nu_cos),0],(N,1))
        t = R.from_rotvec(phi_d)
        t1 = R.from_rotvec(theta_d)
        p = (t.apply(t1.apply(p.T))).T
        return p


    def get_particles_momenta(self,x, y, nu_cos, nu_sin, nu_phi,opts):
        N = 0
        if opts.N_event == 1: 
            pulse = np.zeros([2,3])
            N = 1
        else:
            N = min(opts.N_event,len(x))
            pulse = np.zeros([2,3,N])
        pulse[0][0] = self.dis.particles[0]['Px_GeV']
        pulse[0][1] = self.dis.particles[0]['Py_GeV']
        pulse[0][2] = self.dis.particles[0]['Pz_GeV']
        
        pulse[0] = self.rotate_vectors(pulse[0],nu_phi,nu_cos,N)
        
        pulse[1][0] = self.nu_energy_GeV*np.cos(nu_phi)*nu_sin - pulse[0][0]
        pulse[1][1] = self.nu_energy_GeV*np.sin(nu_phi)*nu_sin - pulse[0][1]
        pulse[1][2] = self.nu_energy_GeV*nu_cos - pulse[0][2]
        
        for i in range(2):
            self.dis.particles[i]['Px_GeV'] = pulse[i][0]
            self.dis.particles[i]['Py_GeV'] = pulse[i][1]
            self.dis.particles[i]['Pz_GeV'] = pulse[i][2]
        
    def get_event_multi(self,opts):
        self.configure(opts)
        nu_cos, nu_sin, nu_phi = self.get_neutrino_direction()
        nu_energy = self.get_neutrino_energy()
        self.get_target_multi()
        mode = self.get_current()
        x,y = self.get_xy_Bjorken_old(mode)
        self.dis.dis_kinematics(self.nu_energy_GeV,x,y,opts.N_event)
        return self.get_particles_momenta(x, y, nu_cos, nu_sin, nu_phi,opts)


    def get_event(self,opts):
        # get neutrino energy
        self.configure(opts)
        nu_cos, nu_sin, nu_phi = self.get_neutrino_direction()
        nu_energy = self.get_neutrino_energy()
        self.get_target()
        mode = self.get_current()
        x,y = self.get_xy_Bjorken_old(mode)
        self.dis.dis_kinematics(self.nu_energy_GeV,x,y,opts.N_event)
        return self.get_particles_momenta(x, y, nu_cos, nu_sin, nu_phi,opts)

    def get_event_fix_en(self,opts):
        #self.dis.init_neutrino(self.nu_pdg)
        self.configure(opts)
        nu_cos, nu_sin, nu_phi = self.get_neutrino_direction()
        nu_energy = self.get_neutrino_energy()
        self.get_target()
        mode = self.get_current()
        x,y,weight,xsec,i = self.get_xy_Bjorken_new(mode,opts)
        self.dis.dis_kinematics(self.nu_energy_GeV,x,y,opts.N_event)
        return self.get_particles_momenta(x, y, nu_cos, nu_sin, nu_phi,opts)

    def next(self):
        return self.get_event()



import numpy as np
from scipy.interpolate import RectBivariateSpline
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import nupropagator.flux.Flux as fff
name = ['Numu_H3a+KM_E2.DAT','Numu_H3a+QGS_E2.dat','Numu_H3a+SIBYLL_E2.dat']
model = ['H3a+KM','H3a+QGS','H3a+SIBYLL']
color = ['k','g','r']
E = np.logspace(1,8,8)
cos = np.linspace(0,1,100)
for j in range(len(E)):
    fig = plt.figure(figsize = (18,12)) 
    for i in range(len(name)): 
        fff1 = fff.Flux('KM',name[i],3)
        a = fff1.get_rect_splain()
        z = np.log10(fff1.I3(cos, E[j]))
        plt.plot(cos,z,label = model[i])
    plt.title(r'$(E_{\nu}$ = '+str('{:.2E}'.format(E[j]))+' GeV)')
    plt.xlabel(r'\cos\theta')
    #plt.yscale('log')
    plt.ylabel(r'$\lg(F_{\nu}/cm^2/srad/s/GeV$)')
    plt.legend()
    plt.savefig('/home/vlad/flux_slice_E='+str('{:.2E}'.format(E[j]))+'GeV.pdf')
    plt.show()




import numpy as np
import plotly.graph_objects as go
from scipy.interpolate import RectBivariateSpline
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import nupropagator.flux.Flux as fff
name = ['Numu_H3a+KM_E2.DAT','Numu_H3a+QGS_E2.dat','Numu_H3a+SIBYLL_E2.dat']
model = ['H3a+KM','H3a+QGS','H3a+SIBYLL']
color = ['k','g','r']
X = np.logspace(1,8,100)
Y = np.linspace(0,1,100)
x,y = np.meshgrid(X,Y)
fig, ax = plt.subplots(subplot_kw={'projection': '3d'})
x = np.log10(x)
y = y
i = 0
fff1 = fff.Flux('KM',name[i],3)
a = fff1.get_rect_splain()
z = np.log10(fff1.I3(Y, X))

fig = go.Figure(data=[go.Surface(z=z, x=x, y=y)])

fig.update_layout(title=model[i], autosize=False,
                  width=1000, height=1000,
                  margin=dict(l=65, r=50, b=65, t=90))

fig.show()




import cross_section.cross as cs
import numpy as np
import time as time
from scipy.integrate import*
import vegas as vegas
import nudisxs.integrand_1dx as i1x
from nudisxs.disxs import *


#def crossy(c):
#    b=quad(lambda x: crossxy(energy,np.exp(x),y,c)*np.exp(x),np.log(dis.xl(energy)[0]),np.log(dis.xl(energy)[1]),epsrel=eps)
    #if b[0]<0:
    #    return 0
    #else:
    #    return b[0]*normfactor





def calc_regfun(ne,ny,y_low,y_up,ratio, nupdg=16,model = 'CT10nlo'):
    #dis = disxs()
    integ = i1x.integrand_1dx()
    integ.dis.init_pdf(name = model)
    integ.dis.init_neutrino(pdg=nupdg)
    integ.init_vegas_integrator()
    e_low=10
    s_p=cs.Cross_section(model,nupdg,2212)
    s_n=cs.Cross_section(model,nupdg,2112)
    e_up=10.**(14)
    funreg=np.zeros([ne,2*ny])
    fullcross_n=np.zeros(ne)
    fullcross_p=np.zeros(ne)
    ycross_n=np.zeros([ne,2*ny])
    ycross_p=np.zeros([ne,2*ny])
    e=np.logspace(np.log10(e_low),np.log10(e_up),num=ne)
    y=np.logspace(np.log10(y_low),-1,num=ny)
    y1=np.linspace(0.11,y_up,num=ny)
    fullcross_p=s_p.ff(e)
    integ.dis.init_current('nc')
    integ.dis.init_target('proton')
    for i in range(ne):
        tt  = time.time()
        for j in range(2*ny):
            if j<ny:
                ycross_p[i][j]=integ.calculate(e[i]/(1-y[j]),y[j])
            else:
                ycross_p[i][j]=integ.calculate(e[i]/(1-y1[j-ny]),y1[j-ny])
        print('p ',i,time.time()-tt)

    integ.dis.init_target('neutron')
    for i in range(ne):
        tt  = time.time()
        for j in range(2*ny):
            if j<ny:
                ycross_n[i][j]=integ.calculate(e[i]/(1-y[j]),y[j])
            else:
                ycross_n[i][j]=integ.calculate(e[i]/(1-y1[j-ny]),y1[j-ny])
        print('n ',i,time.time()-tt)
    fullcross_n=s_n.ff(e)
        
    for i in range(ne):
        for j in range(2*ny):
            funreg[i][j]=(ratio*ycross_p[i][j]+ycross_n[i][j])/(ratio*fullcross_p[i]+fullcross_n[i])

    fil=open('regfun_'+str(model)+'_'+str(nupdg)+'_'+str(ratio)+'.dat','w')
    for i in range(ne):
        for j in range(2*ny):
            if j<ny:
                fil.write(str(e[i])+'\t'+str(y[j])+'\t'+str(funreg[i][j])+'\n')
            else:
                fil.write(str(e[i])+'\t'+str(y1[j-ny])+'\t'+str(funreg[i][j])+'\n')

    fil.close()



calc_regfun(300,200,10**(-16),0.999999999,1.2)



import nupropagator.flux.Flux as fff
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import cm
from types import SimpleNamespace
opts = SimpleNamespace()
opts.neutrinoPrimary_energy_mode = 'random'
opts.neutrinoPrimary_direction_mode = 'r'
opts.neutrinoPrimary_target_mode = 'random'
opts.neutrinoPrimary_current_mode = 'random'
opts.neutrinoPrimary_pdgid = 14
opts.neutrinoPrimary_pdf_model = 'CT10nlo'
opts.N_event = 1000000000
opts.vegas_neval = 1000
opts.flux_indicator = 1
opts.energy_min = 10
opts.energy_max = 10**8
opts.flux_type = 'model_flux'
fff = fff.Flux('KM','Numu_H3a+KM_E2.DAT',3)
Ind = np.array([0,0.5,1,1.5,2])
fig = plt.figure(figsize = (18,12))
opts.flux_type = 'stupid_ind'
a = fff.get_spectra_points(opts)
counts, bins = np.histogram(a, bins=np.logspace(np.log10(opts.energy_min),np.log10(opts.energy_max), 200))
plt.stairs(counts, bins,label = r'ind')
opts.flux_type = 'model_flux'
for i in range(len(Ind)):
    opts.flux_indicator= Ind[i]
    #a = 0
    a = fff.get_spectra_points(opts)
    counts, bins = np.histogram(a, bins=np.logspace(np.log10(opts.energy_min),np.log10(opts.energy_max), 200))
    #counts, bins = np.histogram(a, bins =200)
    plt.stairs(counts, bins,label = r'$\gamma$ = '+str(Ind[i]))
#plt.xscale('log')
plt.yscale('log')
plt.legend(fontsize = 15)
plt.grid(True)
plt.xlabel(r'$E_{\nu}$, GeV',fontsize = 20)
#plt.xlabel(r'$\log_{10}(E_{\nu}/GeV)$',fontsize = 20)


import importlib
path = importlib.util.find_spec('nupropagator').submodule_search_locations[0]
plt.savefig(path+'/flux/model_flux_log1.pdf')
plt.show()




# energy units
GeV = 1.0
MeV = 1e-3*GeV
TeV = 1e3*GeV
PeV = 1e6*GeV

# length units
m   = 1.0
mm   = 1e-3*m
cm  = 1e-2*m
nm  = 1e-9*m
km  = 1e3*m
# time units
ns  = 1
us  = 1e3*ns
s   = 1e9*ns

# velocity
light_velocity_vacuum = 299792458*m/s

# phys constants
alpha_em = 0.0072973525693



import nupropagator.flux.Flux as fff
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import cm
from types import SimpleNamespace

opts = SimpleNamespace()
opts.neutrinoPrimary_energy_mode = 'random'
opts.neutrinoPrimary_direction_mode = 'r'
opts.neutrinoPrimary_target_mode = 'random'
opts.neutrinoPrimary_current_mode = 'random'
opts.neutrinoPrimary_pdgid = 14
opts.neutrinoPrimary_pdf_model = 'CT10nlo'
opts.N_event = 1000000000
opts.vegas_neval = 1000
opts.flux_indicator = 1
opts.energy_min = 10
opts.energy_max = 10**8
opts.flux_type = 'flux_xs_data'
opts.N_protons = 10
opts.N_neutrons = 8
opts.neutrinoPrimary_pdf_model = 'CT10nlo'
opts.neutrinoPrimary_pdgid = 14


N_E = 1000
N_cos = 1000
ff = np.zeros([N_E,N_cos])

fff = fff.Flux('KM','Numu_H3a+KM_E2.DAT',3)
fff.get_rect_splain()
a = fff.get_spectra_points(opts)

fig = plt.figure(figsize = (18,12))
X = np.log10(a[0])
Z = np.log10(a[3])
Y = a[1]
E = fff.energy
cos = fff.cos_theta
ax = fig.add_subplot(projection='3d')
ax.scatter(X,Y,Z,s = 0.5)
x = np.log10(np.logspace(1,8,N_E))
y = np.linspace(0,1,N_cos)
ff = fff.I3(10**x,y)
z = np.log10(ff)
ax.set_xlabel(r'$\lg(E_{\nu}/GeV)$')
ax.set_ylabel(r'$\cos\theta$')
ax.set_zlabel(r'$\lg(F_{\nu}/cm^2/srad/s/GeV)$')
plt.savefig('/home/vlad/flux_xs_vegas.pdf')
plt.show()



import numpy as np
def unit_vector(v):
    v_norm = np.ones(v.shape[0])
    np.sqrt(np.sum(np.power(v,2), axis=1), out=v_norm)
    zero_cond = v_norm==0.
    v_norm[zero_cond] = np.ones(v_norm[zero_cond].shape)
    v = (v.T/v_norm.T).T
    return v

def uniform_random_vector_in_cone(axis,costheta,eps=1e-10):
    # generate orthogonal random vector
    axis = unit_vector(axis)
    u = np.random.rand(axis.shape[0],axis.shape[1])
    ul = np.sum(u*axis,axis=1)
    ul = axis*ul.reshape(axis.shape[0],1)
    u = u - ul
    u = unit_vector(u)
    v = np.cross(u,axis)
    v = unit_vector(v)
    print(costheta,costheta.shape,axis.shape)
    z = np.random.uniform(costheta,1,(axis.shape[0],))
    phi = 2*np.pi*np.random.uniform(0,1,(axis.shape[0],))

    sintheta = np.sqrt(1-z**2)
    cosphi = np.cos(phi)
    sinphi = np.sin(phi)
    v_new = sintheta[:,None]*(cosphi[:,None]*u+sinphi[:,None]*v)+costheta[:,None]*axis

    c = np.sum(v_new*axis,axis=1)
    mask = c < costheta-eps
    if mask.any():
        print('uniform_random_vector_in_cone ',len(c[mask]))
        check1 = np.sum(u*axis,axis=1)
        check2 = np.sum(v*axis,axis=1)
        check3 = np.sum(u*v,axis=1)
        print('axis.shape',axis.shape)
        print('u.shape',u.shape)
        print('v.shape',v.shape)
        print('v_new.shape',v_new.shape)
        print(check1,check2,check3)
        print('costheta',costheta/c-1)

    return v_new


def rotate_vectors(r,dir,dir0):
    from scipy.spatial.transform import Rotation as R
    dir0 = np.array(dir0)
    phi = np.arctan2(dir0[1],dir0[0])
    theta = np.arccos(dir0[2]/np.sqrt(np.sum(dir0*dir0)))
    phi_d = np.tile([0,0,phi],(len(dir[:,0]),1))
    theta_d = np.tile([0,theta,0],(len(dir[:,0]),1))
    t = R.from_rotvec(phi_d)
    t1 = R.from_rotvec(theta_d)
    r = t.apply(t1.apply(r))
    dir = t.apply(t1.apply(dir))
    return r, dir


def translate_vectors(r, r0):
#    a = np.tile(r0, (r.shape[0],r.shape[1],1))
    r = r + r0
    return r, dir


def align_unit_vectors(a,b):
    # find rotation R: R*a = b
    from scipy.spatial.transform import Rotation
    a = unit_vector(a)
    b = unit_vector(b)
    c = np.cross(a,b)
    c = unit_vector(c)
    cosine = np.sum(a*b,axis=1)
    angle = np.arccos(cosine)
#    print(c.shape,angle.shape)
    rot = Rotation.from_rotvec(c*angle[:,None])
    return rot

def uniform_random_vector_in_cone_old(axis,angle):
    from scipy.spatial.transform import Rotation
    # axis = cone center
    # angle = cone angle (in radians)
    # algorithm from https://math.stackexchange.com/questions/56784/generate-a-random-direction-within-a-cone

    # generate random v1 around (0,0,1) on the sphere segment with theta  in (angle,1)
    z = np.random.uniform(np.cos(angle),1,axis.shape[0])
    phi = 2*np.pi*np.random.uniform(0,1,axis.shape[0])
    v1 = np.array([np.sqrt(1-z**2)*np.cos(phi),np.sqrt(1-z**2)*np.sin(phi),z]).T

    axis = unit_vector(axis)
    # make the vector product of cone axis with unit_z = (0,0,1): orth = unit_z x axis
    unit_z = np.array([0.,0.,1.])
    orth = np.cross(unit_z,axis)
    # check
    mask = np.sum(orth*orth,axis=1) != 0.0
    orth = unit_vector(orth)
#    print(orth)
    # rotate generated random vector v1 around orth by angle between unit_z and axis.
    # This way cone axis will be centered on axis instead of (0,0,1)
    cosines = np.sum(axis*unit_z,axis=1)
    axis_angle = np.arccos(cosines).reshape(cosines.shape[0],1)
    rot = Rotation.from_rotvec(axis_angle * orth)
#    print(rot.as_dcm())

    new_v = rot.apply(v1)
#    a = rot.apply(unit_z)
    c = np.sum(new_v*axis,axis=1)
    mask = np.arccos(c)>angle
    if mask.any():
        print('uniform_random_vector_in_cone ',orth)
    return new_v

def generate_cherenkov_spectrum(lambda_min,lambda_max,sample):
    # generate wavelength spectrum 1/lambda^2 in (lambda_min,lambda_max) according to
    # 1/lambda = 1/lambda_min - u*(1/lambda_min-1/lambda_max), where u = uniform in (0,1)
    u = np.random.uniform(0,1,sample)
    x = 1/lambda_min - u*(1/lambda_min-1/lambda_max)
    return 1/x

def searchsorted2d(a,b):
    m,n = a.shape
    max_num = np.maximum(a.max() - a.min(), b.max() - b.min()) + 1
    r = max_num*np.arange(a.shape[0])[:,None]
    p = np.searchsorted( (a+r).ravel(), (b+r).ravel(), side='left' ).reshape(m,-1)
    return p - n*(np.arange(m)[:,None])

def axes_bounds(x,y,z):
    return [axis_bounds(x), axis_bounds(y), axis_bounds(z)]

def axis_bounds(x):
    return [np.min(x),np.max(x)]



import numpy as np
import plotly.graph_objects as go
from scipy.interpolate import RectBivariateSpline
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import nupropagator.flux.Flux as fff
from types import SimpleNamespace

opts = SimpleNamespace()
opts.neutrinoPrimary_energy_mode = 'random'
opts.neutrinoPrimary_direction_mode = 'r'
opts.neutrinoPrimary_target_mode = 'random'
opts.neutrinoPrimary_current_mode = 'random'
opts.neutrinoPrimary_pdgid = 14
opts.neutrinoPrimary_pdf_model = 'CT10nlo'
opts.N_event = 1000000000
opts.vegas_neval = 1000
opts.flux_indicator = 1
opts.energy_min = 10
opts.energy_max = 10**8
opts.flux_type = 'flux_xs_data'
opts.N_protons = 10
opts.N_neutrons = 8
opts.neutrinoPrimary_pdf_model = 'CT10nlo'
opts.neutrinoPrimary_pdgid = 14


name = ['Numu_H3a+KM_E2.DAT']
model = ['H3a+KM']
color = ['k','g','r']
X = np.logspace(1,8,100)
Y = np.linspace(0,1,100)
x,y = np.meshgrid(X,Y)
fig, ax = plt.subplots(subplot_kw={'projection': '3d'})
x = np.log10(x)
y = y
i = 0
fff1 = fff.Flux('KM',name[i],3)
fff1.prepare_total_weight(opts)
a = fff1.get_rect_splain()
z1 = np.log10(fff1.total_weight([X,Y]))
z = np.log10(fff1.I3(Y, X))

ax.plot_wireframe(x,y,z, color=color[0],label= model[i])
ax.plot_wireframe(x,y,z1, color=color[1],label= 'with xs')
ax.set_xlabel(r'$\lg(E_{\nu}/GeV)$')
ax.set_ylabel(r'$\cos\theta$')
ax.set_zlabel(r'$\lg(F_{\nu}/cm^2/srad/s/GeV)$')
plt.legend()
fig.tight_layout()
plt.savefig('/home/vlad/flux_and_xs.pdf')
plt.show()





import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from nudisxs.disxs import *
from scipy.interpolate import interp1d
from scipy.interpolate import interp2d
import math as m
import nupropagator.Global.Global as g
import vegas 
import nupropagator.cross_section.cross as cs

class Flux:
    def __init__(self, model_name, file, dim):
        import importlib
        path = importlib.util.find_spec('nupropagator').submodule_search_locations[0]
        df = pd.read_csv(path + '/flux/flux_data/' + str(file),header=None, sep ='\t')
        self.energy = []
        self.Ncos=11
        if dim == 2:
            self.Ncos=1
        self.NPoint=len(df)
        self.cos_theta = [1-float(j)/(self.Ncos-1.) for j in range(0,self.Ncos,1)]
        self.cos_theta1 = self.cos_theta[::-1] 
        self.flux = np.zeros((self.Ncos,self.NPoint))
        self.model_name = model_name
        self.dim=dim
        self.energy = np.array(df[0])
        for i in range(self.Ncos-1,-1,-1):
            self.flux[i]=np.array(df[i+1])/np.array(df[0])/np.array(df[0])
        return
    
    def prepare_total_weight(self,opts):
        self.N_p = opts.N_protons
        self.N_n = opts.N_neutrons
        model = opts.neutrinoPrimary_pdf_model
        flavor = opts.neutrinoPrimary_pdgid
        self.cs_p=cs.Cross_section(model=model,pdg=flavor,n_tt=2212)
        self.cs_n=cs.Cross_section(model=model,pdg=flavor,n_tt=2112)

    def flux_x(self,E):
        a=0
        for i in range(self.Ncos):
            a+=2*np.pi*self.flux[i]*0.1
        f = interp1d(self.energy, a, kind='cubic')
        return f(E)
    
    def get_3D_data_points(self):
        return self.flux
    
    def get_rect_splain(self):
        from scipy.interpolate import RectBivariateSpline
        E,cos = np.meshgrid(self.energy,self.cos_theta1)
        self.I3 = RectBivariateSpline(self.cos_theta1, self.energy, self.flux)
        return self.I3,E,cos
    
    @vegas.batchintegrand
    def get_vegas_flux(self,xx):
        z = self.I3(xx[:,1],xx[:,0],grid = False)
        return np.array(z)
   
    @vegas.batchintegrand
    def get_vegas_flux1(self,xx):
        z = (self.I3(xx[:,1],10**xx[:,0],grid = False))*10**xx[:,0]*np.log(10)
        return np.array(z)


    def total_weight(self,xx):
        return (self.N_n*self.cs_n.tot(xx[0])+self.N_p*self.cs_p.tot(xx[0]))*self.get_vegas_flux(xx)

    def get_spectra_points(self,opts):
        if opts.flux_type == 'flux_data':#Sinegovsky flux
            #self.integrator = vegas.Integrator([[self.energy.min(),self.energy.max()], [0,1]], nhcube_batch=2000, sync_ran=False)
            self.integrator = vegas.Integrator([[self.energy.min(),self.energy.max()], [0,1]])
            #self.integrator = vegas.Integrator([[np.log10(self.energy.min()),np.log10(self.energy.max())], [0,1]])
            self.result = self.integrator(self.get_vegas_flux, nitn = 10,neval = opts.vegas_neval)
            E = []
            cos = []
            weight = []
            flux = []
            integral = 0
            for x,wgt in self.integrator.random():
                E.append(x[0])
                cos.append(x[1])
                weight.append(wgt)
                flux.append(self.I3(x[1],x[0]))
                integral = integral + wgt*self.I3(x[1],x[0])
            E = np.array(E)
            cos = np.array(cos)
            weight = np.array(weight)
            flux = np.array(flux)
            print(flux[0])
            print('mean', integral)
            print('sp',np.shape(flux),np.shape(weight))
            self.mean = (weight*flux[:,0,0]).sum()
            return E,cos,weight,flux
        N_flux_number = int(np.ceil(opts.N_event/1000))
        eps = opts.energy_min/opts.energy_max
        if opts.flux_type == 'model_flux':#uniform
            g = opts.flux_indicator
            print('N',N_flux_number, g)
            if g == 1:
                E = np.random.random(N_flux_number)
                #print(E,g)
                E = opts.energy_min*(1/eps)**E
                return E
            if g != 1:
                E = np.random.random(N_flux_number)
                E = opts.energy_max*(eps**(1-g)+E*(1 - eps**(1-g)))**(1/(1-g))
                #print(E,g)
                return E
        if opts.flux_type =='stupid_ind':
            E = np.log10(opts.energy_min) -np.log10(eps)*np.random.random(N_flux_number)
            E = 10**(E) 
            return E
        if opts.flux_type == 'flux_xs_data':
            self.prepare_total_weight(opts)
            self.integrator = vegas.Integrator([[self.energy.min(),self.energy.max()], [0,1]], nhcube_batch=2000, sync_ran=False)
            
            result = self.integrator(self.total_weight, nitn = 10,neval = 1000)
            E = []
            cos = []
            weight = []
            flux = []
            for x,wgt in self.integrator.random():
                E.append(x[0])
                cos.append(x[1])
                weight.append(wgt)
                flux.append(self.total_weight(x))
            E = np.array(E)
            cos = np.array(cos)
            weight = np.array(weight)
            flux = np.array(flux)
            return E,cos,weight,flux














import nupropagator.flux.Flux as fff
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import cm
from types import SimpleNamespace

opts = SimpleNamespace()
opts.neutrinoPrimary_energy_mode = 'random'
opts.neutrinoPrimary_direction_mode = 'r'
opts.neutrinoPrimary_target_mode = 'random'
opts.neutrinoPrimary_current_mode = 'random'
opts.neutrinoPrimary_pdgid = 14
opts.neutrinoPrimary_pdf_model = 'CT10nlo'
opts.N_event = 1000000000
opts.vegas_neval = 10000000
opts.flux_indicator = 1
opts.energy_min = 10
opts.energy_max = 10**8
opts.flux_type = 'flux_data'

N_E = 1000
N_cos = 1000
ff = np.zeros([N_E,N_cos])

fff = fff.Flux('KM','Numu_H3a+KM_E2.DAT',3)
fff.get_rect_splain()
a = fff.get_spectra_points(opts)
fig = plt.figure(figsize = (18,12))
X = np.log10(a[0])
Z = np.log10(a[3])
cos = fff.cos_theta
Y = a[1]
E = fff.energy
cos =  np.linspace(0,1,N_cos)
E = np.logspace(1,8,N_E)
print(fff.result.summary(extended = True,weighted = True))
#ff = fff.I3(cos,E)/np.array(fff.result.itn_results[-1].mean())
#print(ff,fff.mean,fff.mean*ff)
xx,yy = np.meshgrid(np.log10(E),cos)
H, xedges, yedges = np.histogram2d(X, Y, bins=[80, 100])
fig = plt.figure(figsize = (18,12))
ff = fff.I3(cos,E)/fff.mean*H.sum()/80
xpos, ypos = np.meshgrid(xedges[:-1] +(xedges[1]-xedges[0])/2, yedges[:-1] + (yedges[1]-yedges[0])/2, indexing="ij")
ax = fig.add_subplot(projection='3d')
ax.plot_wireframe(xpos,ypos,H, color='r')
ax.plot_wireframe(xx,yy,ff, color='g')
ax.set_xlabel(r'$\lg(E_{\nu}/GeV)$')
ax.set_ylabel(r'$\cos\theta$')
ax.set_zlabel(r'$\lg(F_{\nu}/cm^2/srad/s/GeV)$')
plt.savefig('/home/vlad/flux_vegas2d.pdf')
plt.show()
